package com.qmth.themis.business.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.themis.business.bean.admin.*;
import com.qmth.themis.business.cache.bean.ExamRecordDetailBean;
import com.qmth.themis.business.cache.bean.ExamRecordPaperViewBean;
import com.qmth.themis.business.dto.response.MarkResultDto;
import com.qmth.themis.business.entity.TOeExamRecord;
import com.qmth.themis.business.enums.*;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @Description: 考试记录 服务类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
public interface TOeExamRecordService extends IService<TOeExamRecord> {

    /**
     * 获取考试未完列表
     *
     * @param studentId
     * @param examId
     * @param orgId
     * @return
     */
    public Map getUnFinishExam(Long studentId, Long examId, Long orgId);

    Long saveByPrepare(Long examId, Long examActivityId, Long examStudentId, Long paperId, Integer serialNumber, Long studentId);

    /**
     * 计算客观分
     *
     * @param param
     */
//    void calculateObjectiveScore(Map<String, Object> param);

    /**
     * 考试记录数据持久化
     *
     * @param recordId
     */
    public void saveDataByCache(Long recordId);

    void saveFaceVerify(ExamTypeEnum type, Long recordId, Long entryAuthenticationId,
                        VerifyExceptionEnum entryAuthenticationResult);

    void saveLivenessVerify(LivenessTypeEnum type, Long recordId, Long entryAuthenticationId,
                            VerifyExceptionEnum entryAuthenticationResult);

    /**
     * 保存考试记录数据
     *
     * @param recordId
     */
    void examRecordDataSave(Long recordId);

    /**
     * 发送mq更新考试记录
     *
     * @param recordId
     * @param timestamp
     */
    void sendExamRecordDataSaveMq(Long recordId, Long timestamp);

    /**
     * 查询实时监控台列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param paperDownload
     * @param status
     * @param name
     * @param identity
     * @param minWarningCount
     * @param maxWarningCount
     * @param clientWebsocketStatus
     * @param monitorStatusSource
     * @param userId
     * @return
     */
    public IPage<InvigilateListBean> invigilatePageList(IPage<Map> iPage, Long examId,
                                                        Long examActivityId,
                                                        String roomCode,
                                                        Integer paperDownload,
                                                        String status,
                                                        String name,
                                                        String identity,
                                                        Integer minWarningCount,
                                                        Integer maxWarningCount,
                                                        String clientWebsocketStatus,
                                                        String monitorStatusSource,
                                                        Long userId);

    /**
     * 查询实时监控台视频列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param paperDownload
     * @param status
     * @param name
     * @param identity
     * @param minWarningCount
     * @param maxWarningCount
     * @param clientWebsocketStatus
     * @param cameraMonitorStatus
     * @param screenMonitorStatus
     * @param mobileFirstMonitorStatus
     * @param mobileSecondMonitorStatus
     * @param userId
     * @param orgId
     * @return
     */
    public IPage<InvigilateListVideoBean> invigilatePageListVideo(IPage<Map> iPage, Long examId,
                                                                  Long examActivityId,
                                                                  String roomCode,
                                                                  Integer paperDownload,
                                                                  String status,
                                                                  String name,
                                                                  String identity,
                                                                  Integer minWarningCount,
                                                                  Integer maxWarningCount,
                                                                  String clientWebsocketStatus,
                                                                  String cameraMonitorStatus,
                                                                  String screenMonitorStatus,
                                                                  String mobileFirstMonitorStatus,
                                                                  String mobileSecondMonitorStatus,
                                                                  Long userId,
                                                                  Long orgId);

    /**
     * 实时监控台视频随机列表
     *
     * @param examId
     * @param userId
     * @param randomNum
     * @param orgId
     * @return
     */
    public List<InvigilateListVideoBean> invigilatePageListVideoRandom(Long examId, Long userId, Integer randomNum, Long orgId);

    /**
     * 查询在线巡考列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param status
     * @param name
     * @param identity
     * @param minMultipleFaceCount
     * @param maxMultipleFaceCount
     * @param minExceptionCount
     * @param maxExceptionCount
     * @param minWarningCount
     * @param maxWarningCount
     * @param clientWebsocketStatus
     * @param userId
     * @param orgId
     * @return
     */
    public IPage<InvigilateListPatrolBean> invigilatePagePatrolList(IPage<Map> iPage, Long examId,
                                                                    Long examActivityId,
                                                                    String roomCode,
                                                                    String status,
                                                                    String name,
                                                                    String identity,
                                                                    Integer minMultipleFaceCount,
                                                                    Integer maxMultipleFaceCount,
                                                                    Integer minExceptionCount,
                                                                    Integer maxExceptionCount,
                                                                    Integer minWarningCount,
                                                                    Integer maxWarningCount,
                                                                    String clientWebsocketStatus,
                                                                    Long userId,
                                                                    Long orgId);

    /**
     * 预警提醒列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param approveStatus
     * @param name
     * @param identity
     * @param minMultipleFaceCount
     * @param maxMultipleFaceCount
     * @param minExceptionCount
     * @param maxExceptionCount
     * @param minWarningCount
     * @param maxWarningCount
     * @param userId
     * @param orgId
     * @return
     */
    public IPage<InvigilateListWarningBean> invigilatePageWarningList(IPage<Map> iPage, Long examId,
                                                                      Long examActivityId,
                                                                      String roomCode,
                                                                      Integer approveStatus,
                                                                      String name,
                                                                      String identity,
                                                                      Integer minMultipleFaceCount,
                                                                      Integer maxMultipleFaceCount,
                                                                      Integer minExceptionCount,
                                                                      Integer maxExceptionCount,
                                                                      Integer minWarningCount,
                                                                      Integer maxWarningCount,
                                                                      Long userId,
                                                                      Long orgId);

    /**
     * 预警提醒清除未阅列表
     *
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param approveStatus
     * @param name
     * @param identity
     * @param minMultipleFaceCount
     * @param maxMultipleFaceCount
     * @param minExceptionCount
     * @param maxExceptionCount
     * @param minWarningCount
     * @param maxWarningCount
     * @param userId
     * @param orgId
     * @return
     */
    public Integer approveStatusListUpdate(Long examId,
                                           Long examActivityId,
                                           String roomCode,
                                           Integer approveStatus,
                                           String name,
                                           String identity,
                                           Integer minMultipleFaceCount,
                                           Integer maxMultipleFaceCount,
                                           Integer minExceptionCount,
                                           Integer maxExceptionCount,
                                           Integer minWarningCount,
                                           Integer maxWarningCount,
                                           Long userId,
                                           Long orgId);

    /**
     * 进度查询列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param courseCode
     * @param name
     * @param identity
     * @param status
     * @param userId
     * @param orgId
     * @return
     */
    public IPage<InvigilateListProgressBean> invigilatePageProgressList(IPage<Map> iPage, Long examId,
                                                                        Long examActivityId,
                                                                        String roomCode,
                                                                        String courseCode,
                                                                        String name,
                                                                        String identity,
                                                                        Integer status,
                                                                        Long userId,
                                                                        Long orgId);

    /**
     * 进度查询列表导出
     *
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param courseCode
     * @param name
     * @param identity
     * @param status
     * @param userId
     * @param orgId
     * @return
     */
    public List<InvigilateListProgressExcelBean> invigilatePageProgressListExport(Long examId,
                                                                                  Long examActivityId,
                                                                                  String roomCode,
                                                                                  String courseCode,
                                                                                  String name,
                                                                                  String identity,
                                                                                  Integer status,
                                                                                  Long userId,
                                                                                  Long orgId);

    /**
     * 监考明细管理列表
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param courseCode
     * @param status
     * @param breachStatus
     * @param finishType
     * @param name
     * @param identity
     * @param minMultipleFaceCount
     * @param maxMultipleFaceCount
     * @param minExceptionCount
     * @param maxExceptionCount
     * @param minWarningCount
     * @param maxWarningCount
     * @param userId
     * @param orgId
     * @return
     */
    public IPage<InvigilateListHistoryBean> invigilatePageListHistory(IPage<Map> iPage, Long examId,
                                                                      Long examActivityId,
                                                                      String roomCode,
                                                                      String courseCode,
                                                                      String status,
                                                                      Integer breachStatus,
                                                                      String finishType,
                                                                      String name,
                                                                      String identity,
                                                                      Integer minMultipleFaceCount,
                                                                      Integer maxMultipleFaceCount,
                                                                      Integer minExceptionCount,
                                                                      Integer maxExceptionCount,
                                                                      Integer minWarningCount,
                                                                      Integer maxWarningCount,
                                                                      Long userId,
                                                                      Long orgId);

    /**
     * 监考明细管理列表导出
     *
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param courseCode
     * @param status
     * @param breachStatus
     * @param finishType
     * @param name
     * @param identity
     * @param minMultipleFaceCount
     * @param maxMultipleFaceCount
     * @param minExceptionCount
     * @param maxExceptionCount
     * @param minWarningCount
     * @param maxWarningCount
     * @param userId
     * @param orgId
     * @return
     */
    public List<InvigilateListHistoryBean> invigilatePageListHistoryExport(Long examId,
                                                                           Long examActivityId,
                                                                           String roomCode,
                                                                           String courseCode,
                                                                           String status,
                                                                           Integer breachStatus,
                                                                           String finishType,
                                                                           String name,
                                                                           String identity,
                                                                           Integer minMultipleFaceCount,
                                                                           Integer maxMultipleFaceCount,
                                                                           Integer minExceptionCount,
                                                                           Integer maxExceptionCount,
                                                                           Integer minWarningCount,
                                                                           Integer maxWarningCount,
                                                                           Long userId,
                                                                           Long orgId);


    /**
     * 重新算分
     */
    void calculateScore(Long recordId);

    /**
     * 获取考试记录数
     *
     * @param examId
     * @return
     */
    Long getCountByExamId(Long examId);

    /**
     * 批量获取考试记录
     *
     * @param examId
     * @param startId
     * @return
     */
    List<TOeExamRecord> getListByExamIdAndStartId(Long examId, Long startId);

    TOeExamRecord findOneByPaperId(Long paperId);

    TOeExamRecord findOneByExamId(Long examId, ExamRecordStatusEnum status);

    public List<OpenRecordNeedMarkBean> findExamRecordNeedMark(Long examStudentId);

    /**
     * 寻找客观分最高
     *
     * @param examId
     * @param examStudentId
     * @return
     */
    List<TOeExamRecord> findMaxObjectiveScore(Long examId, Long examStudentId);

    /**
     * 查询成绩
     *
     * @param markResultDto
     * @param recordSelectStrategyEnum
     * @return
     */
    TOeExamRecord findMarkResult(MarkResultDto markResultDto, RecordSelectStrategyEnum recordSelectStrategyEnum);

    /**
     * 设置断点信息
     *
     * @param recordId
     */
    Boolean setExamBreak(Long recordId);

    /**
     * 断点逻辑
     *
     * @param recordId
     * @param setBreak
     * @return
     */
    Boolean examBreakLogic(Long recordId, boolean setBreak);

    /**
     * 根据orgId查找 在线人数/已待考/考试人数
     *
     * @param orgId
     * @return
     */
    List<Long> findByOrgId(Long orgId);

    /**
     * 查找监控消息
     *
     * @param examId
     * @param roomCodeSet
     * @return
     */
    List<CountStopBean> findByStopInfo(Long examId, Set<String> roomCodeSet);

    /**
     * 查找在线消息
     *
     * @param examId
     * @param roomCodeSet
     * @return
     */
    List<Integer> findByOnlineInfo(Long examId, Set<String> roomCodeSet);

    /**
     * 查找在线消息
     *
     * @param examId
     * @param roomCodeSet
     * @return
     */
    List<TOeExamRecord> findByOnlineInfoRecord(Long examId, Set<String> roomCodeSet);

    /**
     * 监考结束
     *
     * @param examId
     * @param roomCodeSet
     * @return
     */
    List<Integer> findByFinish(Long examId, Set<String> roomCodeSet);

    /**
     * 管理员报表
     *
     * @param orgId
     * @param examId
     * @param examActivityId
     * @param roomCode
     * @param courseCode
     * @param monitorStatus
     * @return
     */
    List<Integer> findByViewAdmin(Long orgId, Long examId, Long examActivityId, String roomCode, String courseCode, InvigilateMonitorStatusEnum monitorStatus);

    /**
     * 考试明细查询接口
     *
     * @param iPage
     * @param examId
     * @param examActivityId
     * @param courseCode
     * @param roomCode
     * @param finishType
     * @param name
     * @param identity
     * @return
     */
    public IPage<ExamRecordDetailBean> examRecordDetailQuery(IPage<Map> iPage,
                                                             Long examId,
                                                             Long examActivityId,
                                                             String courseCode,
                                                             String roomCode,
                                                             String finishType,
                                                             String name,
                                                             String identity);

    /**
     * 考试明细查询接口
     *
     * @param examId
     * @param courseCodeList
     * @param identityList
     * @return
     */
    public List<String> examRecordDetailQuery(Long examId, List<String> courseCodeList, List<String> identityList);

    /**
     * 查看试卷
     *
     * @param examRecordId
     * @return
     */
    public ExamRecordPaperViewBean examRecordPaperView(Long examRecordId) throws IOException;
}