package com.qmth.themis.business.service.impl;

import com.aliyun.oss.common.utils.BinaryUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import com.qmth.themis.business.bean.admin.StudentPhotoUploadResponseBean;
import com.qmth.themis.business.dao.TEStudentMapper;
import com.qmth.themis.business.dto.response.TEStudentDto;
import com.qmth.themis.business.dto.response.TEStudentExamRecordDto;
import com.qmth.themis.business.dto.response.TEStudentMonitorRecordDto;
import com.qmth.themis.business.entity.TEStudent;
import com.qmth.themis.business.service.CacheService;
import com.qmth.themis.business.service.TEStudentService;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.OssUtil;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.HexUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Map;

/**
 * @Description: 学生档案 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TEStudentServiceImpl extends ServiceImpl<TEStudentMapper, TEStudent> implements TEStudentService {

    @Resource
    TEStudentMapper teStudentMapper;

    @Resource
    OssUtil ossUtil;

    @Resource
    CacheService cacheService;

    /**
     * 学生档案查询
     *
     * @param iPage
     * @param identity
     * @param name
     * @param enable
     * @param orgId
     * @param photo
     * @return
     */
    @Override
    public IPage<TEStudentDto> studentQuery(IPage<Map> iPage, String identity, String name, Integer enable,
                                            Long orgId, Boolean photo) {
        return teStudentMapper.studentQuery(iPage, identity, name, enable, orgId, photo);
    }

    /**
     * 学生考试记录查询
     *
     * @param iPage
     * @param studentId
     * @param examId
     * @return
     */
    @Override
    public IPage<TEStudentExamRecordDto> studentExamRecordQuery(IPage<Map> iPage, Long studentId, Long examId) {
        IPage<TEStudentExamRecordDto> teStudentExamRecordDtoIPage = teStudentMapper.studentExamRecordQuery(iPage, studentId, examId);
        Gson gson = new Gson();
        for (TEStudentExamRecordDto t : teStudentExamRecordDtoIPage.getRecords()) {
            List<TEStudentMonitorRecordDto> monitorRecordList = gson.fromJson(t.getTencentVideoUrl(), new TypeToken<List<TEStudentMonitorRecordDto>>() {
            }.getType());
            t.setMonitorRecord(monitorRecordList);
        }
        return teStudentExamRecordDtoIPage;
    }

    @Transactional
    @Override
    public StudentPhotoUploadResponseBean photoUpload(Long orgId, String identity, MultipartFile file, String md5) {
        QueryWrapper<TEStudent> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(TEStudent::getOrgId, orgId);
        wrapper.lambda().eq(TEStudent::getIdentity, identity);
        TEStudent student = this.getOne(wrapper);
        if (student == null) {
            throw new BusinessException("学生不存在");
        }
        String fileName = file.getOriginalFilename();
        String suffix = fileName.substring(fileName.lastIndexOf(".") + 1).toLowerCase();
        String filePath = "base_photo/" + orgId + "/" + student.getId() + "." + suffix;
        InputStream in = null;
        try {
            student.setBasePhotoPath(filePath);
            this.saveOrUpdate(student);
            cacheService.updateStudentAccountCache(student.getId());
            in = file.getInputStream();
            ossUtil.upload(true, filePath, in, BinaryUtil.toBase64String(HexUtils.decodeHex(md5)));
            String url = ossUtil.getAliYunOssPublicDomain().getPublicUrl() + File.separator + filePath;
            StudentPhotoUploadResponseBean ret = new StudentPhotoUploadResponseBean();
            ret.setPhotoUrl(url);
            return ret;
        } catch (IOException e) {
            throw new BusinessException("文件读取出错");
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                }
            }
        }
    }
}
