package com.qmth.themis.business.templete.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dao.TSyncExamStudentScoreMapper;
import com.qmth.themis.business.entity.TSyncExamStudentScore;
import com.qmth.themis.business.enums.TaskStatusEnum;
import com.qmth.themis.business.service.TSyncExamStudentScoreService;
import com.qmth.themis.business.templete.TaskSyncCommon;
import com.qmth.themis.business.templete.TaskSyncTemplete;
import com.qmth.themis.business.util.CloudMarkUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;

/**
 * @Description: 同步云阅卷成绩任务
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/10/20
 */
@Service
public class TaskSyncMarkCloudTemplete implements TaskSyncTemplete {

    private final static Logger log = LoggerFactory.getLogger(TaskSyncMarkCloudTemplete.class);

    @Resource
    CloudMarkUtil cloudMarkUtil;

    @Resource
    TSyncExamStudentScoreService tSyncExamStudentScoreService;

    @Resource
    TSyncExamStudentScoreMapper tSyncExamStudentScoreMapper;

    /**
     * 同步模版
     *
     * @param map
     * @return
     */
    @Override
    public Result syncTask(Map<String, Object> map) throws IOException {
        log.info("开始同步云阅卷成绩数据");
        long start = System.currentTimeMillis();
        TaskSyncCommon taskSyncCommon = new TaskSyncCommon(map);
        taskSyncCommon.init();
        taskSyncCommon.setTxtList(new ArrayList());
        StringJoiner stringJoiner = null;
        try {
            taskSyncCommon.getTxtList()
                    .add(DateUtil.format(new Date(), Constants.DEFAULT_DATE_PATTERN) + "->开始准备同步云阅卷成绩数据");
            Object o = map.get("yunMarkExamId");
            Long yunMarkExamId = null;
            if (o instanceof Long) {
                yunMarkExamId = (Long) o;
            } else if (o instanceof Integer) {
                yunMarkExamId = Long.parseLong(String.valueOf(o));
            }
            taskSyncCommon.getTbTaskHistory().setEntityId(yunMarkExamId);
            Long examId = (Long) map.get(SystemConstant.EXAM_ID);
            Long orgId = (Long) map.get(SystemConstant.ORG_ID);
            Long createId = (Long) map.get(SystemConstant.CREATE_ID);
            String examCode = (String) map.get(SystemConstant.EXAM_CODE);

            boolean oss = (boolean) taskSyncCommon.getOssEnv().get(SystemConstant.OSS);
            stringJoiner = new StringJoiner("");
            if (!oss) {
                stringJoiner.add(SystemConstant.FILES_DIR).add(File.separator);
            }
            LocalDateTime nowTime = LocalDateTime.now();
            stringJoiner.add(String.valueOf(nowTime.getYear())).add(File.separator)
                    .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                    .add(String.format("%02d", nowTime.getDayOfMonth()));
            taskSyncCommon.getTbTaskHistory().setStatus(TaskStatusEnum.RUNNING);
            taskSyncCommon.getTbTaskHistory().setSummary("正在同步数据");
            taskSyncCommon.getTxtList().add(DateUtil.format(new Date(), Constants.DEFAULT_DATE_PATTERN) + "->" + taskSyncCommon.getTbTaskHistory().getSummary());
            List<Map> finalList = cloudMarkUtil.callStudentScore(orgId, yunMarkExamId, examCode, String.valueOf(map.get(SystemConstant.ACCESS_KEY_ID)), String.valueOf(map.get(SystemConstant.ACCESS_KEY_SECRET)));
            if (!CollectionUtils.isEmpty(finalList)) {
                List<TSyncExamStudentScore> tSyncExamStudentScoreList = new ArrayList<>(finalList.size());
                for (Map student : finalList) {
                    student.put("exam_batch_id", examId);
                    student.put(SystemConstant.CREATE_ID, createId);
                    tSyncExamStudentScoreList.add(new TSyncExamStudentScore(student));
                }
                if (tSyncExamStudentScoreList.size() > 0) {
                    tSyncExamStudentScoreService.remove(new QueryWrapper<TSyncExamStudentScore>().lambda().eq(TSyncExamStudentScore::getExamBatchId, examId).eq(TSyncExamStudentScore::getExamId, yunMarkExamId));
                    tSyncExamStudentScoreMapper.insertBatch(tSyncExamStudentScoreList);
                }

                taskSyncCommon.getTbTaskHistory().setStatus(TaskStatusEnum.FINISH);
                taskSyncCommon.getTbTaskHistory().setSummary("同步了" + finalList.size() + "条数据");
                taskSyncCommon.getTbTaskHistory().setProgress(100D);
                taskSyncCommon.getTxtList().add(DateUtil.format(new Date(), Constants.DEFAULT_DATE_PATTERN) + "->" + taskSyncCommon.getTbTaskHistory().getSummary());
            }
            long end = System.currentTimeMillis();
            log.info("同步云阅卷成绩数据结束,============耗时============:{}秒", (end - start) / 1000);
        } catch (Exception e) {
            log.error("同步云阅卷成绩数据报错", e);
            taskSyncCommon.exception(e, taskSyncCommon.getTxtList());
        } finally {
            //这里写入txt文件
            taskSyncCommon.writeSyncResultTxt(stringJoiner.toString(), taskSyncCommon.getTxtList().toString());
        }
        return ResultUtil.ok(true);
    }
}