package com.qmth.themis.business.util;

import com.alibaba.fastjson.JSONObject;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.domain.YunMarkDomain;
import com.qmth.themis.business.entity.TBOrg;
import com.qmth.themis.business.service.CacheService;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.signature.SignatureInfo;
import com.qmth.themis.common.signature.SignatureType;
import org.apache.commons.lang3.StringEscapeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;

/**
 * 同步云阅卷接口工具类
 * <p>
 * Date: 2021/10/29.
 */
@Component
public class CloudMarkUtil {
    private final static Logger log = LoggerFactory.getLogger(CloudMarkUtil.class);

    @Resource
    CacheService cacheService;

    private YunMarkDomain yunMarkDomain;

    @Bean
    public YunMarkDomain yunMarkEnv(YunMarkDomain yunMarkDomain) {
        this.yunMarkDomain = new YunMarkDomain(yunMarkDomain.getUrl(), yunMarkDomain.getStudentScoreApi());
        return this.yunMarkDomain;
    }

    public YunMarkDomain getYunMarkDomain() {
        return yunMarkDomain;
    }

    /**
     * 考试成绩考生查询接口
     *
     * @param orgId
     * @param examId
     * @param examCode
     * @param accessKey
     * @param accessSecret
     * @return
     * @throws IOException
     */
    public List<Map> callStudentScore(Long orgId, Long examId, String examCode, String accessKey, String accessSecret) throws IOException {
        String url = yunMarkDomain.getUrl() + yunMarkDomain.getStudentScoreApi();
        Map<String, Object> params = new HashMap<>();
        if (Objects.nonNull(examId)) {
            params.put("examId", examId);
        } else if (Objects.nonNull(examCode)) {
            params.put("examCode", examCode);
        }
        TBOrg tbOrg = cacheService.addOrgCache(orgId);
        Optional.ofNullable(tbOrg).orElseThrow(() -> new BusinessException(ExceptionResultEnum.ORG_NO));

        if (Objects.isNull(accessKey) || Objects.equals("", accessKey.trim())) {
            accessKey = tbOrg.getAccessKey();
        }
        if (Objects.isNull(accessSecret) || Objects.equals("", accessSecret.trim())) {
            accessSecret = tbOrg.getAccessSecret();
        }
        int pageNumber = 1;
        int pageSize = 100;
        List<Map> students = null;
        for (; ; ) {
            params.put("pageNumber", pageNumber);
            params.put("pageSize", pageSize);
            Long timestamp = System.currentTimeMillis();
            String accessToken = SignatureInfo.build(SignatureType.SECRET, SystemConstant.METHOD, yunMarkDomain.getStudentScoreApi(), timestamp, accessKey, accessSecret);
            String result = HttpUtil.post(url, params, accessToken, timestamp);
            result = StringEscapeUtils.unescapeHtml4(result);
            if (Objects.nonNull(result)) {
                if (result.contains("HTTP")) {
                    throw new BusinessException("云阅卷鉴权失败");
                }
                List<Map> tempList = JSONObject.parseArray(JSONObject.toJSON(result).toString(), Map.class);
                if (Objects.nonNull(tempList) && tempList.size() > 0) {
                    if (Objects.isNull(students)) {
                        students = new ArrayList<>();
                    }
                    students.addAll(tempList);
                    pageNumber++;
                } else {
                    break;
                }
            } else {
                break;
            }
        }
        return students;
    }
}
